/**************************************************************
 *      Template zur Ansteuerung des Display SDA5707
 *      ============================================
 *                   Uwe Berger; 2009   
 * 
 * 
 * 
 * 
 * --> Hardware-Protokoll-Beschreibung des Display SDA5708: 
 * Quelle: http://www.sbprojects.com/knowledge/footprints/sda5708.htm 
 *  
 * 
 * SDA5708 Pins (Pin 1 is right of the back side)
 * -------------------------------------------------
 * Pin Function
 * 1   Vcc
 * 2   LOAD
 * 3   DATA
 * 4   SDCLK
 * 5   RESET
 * 6   GND
 * 
 * 
 * SDA5708 Datagramm
 * -----------------
 * 
 * LOAD  ---+                          +---
 *          |                          |
 *          +--------------------------+
 * DATA  ----+----+----+----+----+----+----+----+----+-----
 *           | D0 | D1 | D2 | D3 | D4 | D5 | D6 | D7 |
 *       ----+----+----+----+----+----+----+----+----+-----
 * SDCLK --+    +--+ +--+ +--+ +--+ +--+ +--+ +--+ +--+
 *         |    |  | |  | |  | |  | |  | |  | |  | |  | 
 *         +----+  +-+  +-+  +-+  +-+  +-+  +-+  +-+  +-----
 * 
 * 
 * SDA5708 Register
 * ----------------
 * 
 * SDA5708 --> Control-Register
 * 76543210
 * ||||||||
 * |||||000 --> 100% Brightness
 * |||||001 --> 53%
 * |||||010 --> 40%
 * |||||011 --> 27%
 * |||||100 --> 20%
 * |||||101 --> 13%
 * |||||110 --> 6.6%
 * |||||111 --> 0% (dark ;-))
 * ||||+------> 0=max. peak current; 1=12.5% of max. peak current
 * |||0
 * ||+--------> 0=clear display; 1=normal
 * 11
 * 
 * 
 * SDA5807 --> Adress-Register (Digit 0 left; Digit 7 right)
 * 76543210
 * ||||||||
 * |||||000 --> Digit 0
 * |||||001 --> Digit 1
 * |||||010 --> ...
 * |||||111 --> Digit 7
 * 10100
 * 
 * ---------
 * Have fun!
 * 
 ***************************************************************/ 
#include <avr/pgmspace.h>
#include "font.h" 

// Hardwarekonfiguration MCU
#define PORT_RESET	PORTC
#define DDR_RESET	DDRC
#define RESET 		PC2

#define PORT_SDCLK	PORTC
#define DDR_SDCLK	DDRC
#define SDCLK 		PC0

#define PORT_DATA	PORTC
#define DDR_DATA	DDRC
#define DATA 		PC1

#define PORT_LOAD	PORTC
#define DDR_LOAD	DDRC
#define LOAD 		PC3

//*********************************************************************
void reset_SDA5708(void)
{
	// RESET toggeln
	PORT_RESET &= ~(1<<RESET);
	PORT_RESET |= (1<<RESET);
}

//*********************************************************************
void init_SDA5708(void)
{
	// Ausgaenge initialisieren
	DDR_RESET |= (1<<RESET);
	DDR_SDCLK |= (1<<SDCLK);
	DDR_DATA  |= (1<<DATA);
	DDR_LOAD  |= (1<<LOAD);
	reset_SDA5708();
}

//*********************************************************************
void send_byte_to_SDA5708(uint8_t byte)
{
	uint8_t x;	
	// LOAD auf Low
	PORT_LOAD &= ~(1<<LOAD);
	// jede Byte-Stelle ans Display senden
	for (x=0; x<=7;x++)	{
		// DATA entsprechend setzen
		if ((byte>>x)&1) PORT_DATA |= (1<<DATA); else PORT_DATA &= ~(1<<DATA);
		// SDCLK toggeln
		PORT_SDCLK |= (1<<SDCLK);
		PORT_SDCLK &= ~(1<<SDCLK);
	}
	// LOAD auf High
	PORT_LOAD |= (1<<LOAD);
}


//*********************************************************************
void brightness_SDA5708(uint8_t val)
{
	// Helligkeit von 0...8, wobei 0 ganz hell ist!
	// mit dem 4.Bit koennte man noch die Multiplex-Impulslaenge des 
	// Display-Controllere manipulieren und damit das Ding noch dunkler
	// machen... (siehe auch Hardware-/Protokoll-Beschreibung oben!)
	send_byte_to_SDA5708(0b11100000|(val&0b00000111));
}

//*********************************************************************
void digit_to_SDA5708(uint8_t sign, uint8_t digit)
{
	uint8_t i;
	// anzeigbares Zeichen? Wenn nein, dann Leerzeichen!
	if ((sign < 0x20) || (sign > 0x7b)) sign = 0x20;
	// Digit im Bereich?
	if (digit > 7) digit = 0;
	// Digit-Adresse zum Display senden
	send_byte_to_SDA5708(0b10100000|digit);
	// jede Zeile des Fonts adressieren, lesen, ausgeben
	for(i=0; i<7; i++) {
		// Font ist linksbuendig, deshalb 3 Bit nach rechts schieben...
		send_byte_to_SDA5708(pgm_read_byte(&font[(sign-0x20)*7+i])/8);
	}
}

//*********************************************************************
void print2display(char *text)
{
	uint8_t cursor=0;
	char *p=text;
	while (*p) {
		digit_to_SDA5708(*p, cursor);
		cursor++;
		p++;
	}
}

//*********************************************************************
//*********************************************************************
//*********************************************************************
int main(void)
{

	// Port-Initialisierung, Display-Reset
	init_SDA5708();
  
	// Helligkeit einstellen 
	brightness_SDA5708(2);

	// Zeichenkette ausgeben
	print2display("Boerges!\0");
	
	while(1);
}
